# SPDX-FileCopyrightText: 2023 2022-2023 Abilian SAS <https://abilian.com/>
#
# SPDX-License-Identifier: MIT
import shlex
import subprocess
import sys

import typer

app = typer.Typer()


def run(cmd, echo=True, warn=False):
    if echo:
        typer.secho("> " + cmd, fg=typer.colors.GREEN)
    args = shlex.split(cmd)
    p = subprocess.Popen(args)
    p.wait()
    if not warn and p.returncode:
        typer.secho(f"failed with error code {p.returncode}", fg="red")
        # typer.secho(p.stderr)
        sys.exit()
    return p.returncode



# SPDX-FileCopyrightText: 2023 Abilian SAS <https://abilian.com/>
#
# SPDX-License-Identifier: MIT

import typer

# Import submodules to register their commands
from . import bumper, commands, help
from .app import app

assert bumper
assert help
assert commands


@app.callback(invoke_without_command=True)
def main(ctx: typer.Context):
    """Abilian Dev Tools command-line runner.

    Helps keeping your project clean and healthy.
    """
    if ctx.invoked_subcommand is None:
        ctx.get_help()



"""Version bumpers, using date-based versioning.

Todo: also support semantic versionning.
"""

import sys
from time import gmtime, strftime

import tomlkit
from invoke import Context

TASKS = ["bump_version"]


def bump_version(c: Context, rule: str = "patch"):
    """Bump version in pyproject.toml, commit & apply tag.

    Parameter "rule" can be one of: "daily" or a parameter accepted by
    "poetry version".
    """
    r = c.run("git diff --quiet", echo=True, warn=True)
    if r.exited != 0:
        print("git diff --quiet failed")
        sys.exit()

    update_version(c, rule)
    version = get_version()
    c.run(f"git tag {version}", echo=True)
    c.run("git add pyproject.toml", echo=True)
    c.run(f"git commit -m 'Bump version ({version})'", echo=True)


def update_version(c, rule):
    if rule == "daily":
        update_version_daily(rule)
    else:
        c.run(f"poetry version {rule}", echo=True)


def update_version_daily(rule):
    pyproject_d = tomlkit.parse(open("pyproject.toml").read())
    version: str = pyproject_d["tool"]["poetry"]["version"]  # type: ignore
    serial = int(version.split(".")[-1])
    date = version[: -len(str(serial)) - 1]

    new_date = strftime("%Y.%m.%d", gmtime())
    if date == new_date:
        serial += 1
    else:
        serial = 1

    new_version = f"{new_date}.{serial}"
    print("New version: ", new_version)
    pyproject_d["tool"]["poetry"]["version"] = new_version  # type: ignore
    with open("pyproject.toml", "w") as f:
        f.write(tomlkit.dumps(pyproject_d))


def get_version():
    d = tomlkit.parse(open("pyproject.toml").read())
    version: str = d["tool"]["poetry"]["version"]  # type: ignore
    return version



# SPDX-FileCopyrightText: 2023 Abilian SAS <https://abilian.com/>
#
# SPDX-License-Identifier: MIT
import glob
import shutil
from pathlib import Path

import typer

from .app import app, run

CRUFT_DIRS = [".mypy_cache", ".pytest_cache", ".ruff_cache"]


@app.command()
def all():
    """Run everything (linters and tests)."""
    check([])
    test([])


@app.command()
def test(args: list[str] = typer.Argument(None)):
    """Run tests."""
    if not args:
        args = ["src", "tests"]

    check_files_exist(args)

    args_str = " ".join(args)
    run(f"pytest {args_str}")


@app.command()
def check(args: list[str] = typer.Argument(None)):
    """Run checker/linters on specified files or directories."""
    if not args:
        args = ["src", "tests"]

    check_files_exist(args)

    args_str = " ".join(args)

    run(f"ruff {args_str}")
    run(f"flake8 {args_str}")
    run(f"mypy --show-error-codes {args_str}")
    run("pyright")
    run(f"vulture --min-confidence 80 {args_str}")
    # TODO: currently broken
    # run("deptry .")


@app.command()
def format(args: list[str] = typer.Argument(None)):
    """Format code in specified files or directories."""
    if not args:
        args = ["src", "tests"]

    check_files_exist(args)

    args_str = " ".join(args)

    run(f"black {args_str}")
    run(f"isort {args_str}")


def check_files_exist(args):
    for arg in args:
        if not Path(arg).exists():
            typer.secho(f"{arg} does not exist", fg=typer.colors.RED)
            raise typer.Exit(1)


@app.command("security-check")
def security_check(ctx: typer.Context):
    """Run security checks (deprecated, use 'audit' instead)."""
    typer.secho(
        "WARNING: 'security-check' is deprecated, use 'audit' instead.",
        fg=typer.colors.YELLOW,
    )
    ctx.invoke(audit)


@app.command()
def audit():
    """Run security audit."""
    run("pip-audit")
    # TODO: don't assume source dir is src
    run("bandit -q -c pyproject.toml -r src")
    # TODO: suppress output on success
    run("reuse lint")
    # TODO: Don't run safety check for now, it's too noisy
    # run("safety check")


@app.command()
def clean():
    """Cleanup cruft."""
    typer.secho(
        "Removing Python bytecode cache directories...", fg=typer.colors.BRIGHT_MAGENTA
    )
    for cache_dir in glob.glob("**/__pycache__", recursive=True):
        shutil.rmtree(cache_dir)

    typer.secho("Removing other caches...", fg=typer.colors.BRIGHT_MAGENTA)
    for cache_dir in CRUFT_DIRS:
        if Path(cache_dir).exists():
            typer.secho(f"Removing {cache_dir}", fg=typer.colors.YELLOW)
            shutil.rmtree(cache_dir, ignore_errors=True)
